
// p4fliconv: high resolution interlaced FLI converter utility
// Copyright (C) 2007-2008 Istvan Varga <istvanv@users.sourceforge.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// The Plus/4 program files generated by this utility are not covered by the
// GNU General Public License, and can be used, modified, and distributed
// without any restrictions.

#ifndef P4FLICONV_MCCHAR_HPP
#define P4FLICONV_MCCHAR_HPP

#include "p4fliconv.hpp"

namespace Plus4FLIConv {

  class P4FLI_MultiColorChar : public FLIConverter {
   public:
    class Line128 {
     private:
      float   *buf;
     public:
      Line128();
      Line128(const Line128& r);
      virtual ~Line128();
      Line128& operator=(const Line128& r);
      void clear();
      inline float& operator[](long n)
      {
        return buf[n];
      }
      inline float& pixel(long x)
      {
        return buf[x];
      }
      inline float getPixel(long x) const
      {
        return buf[x];
      }
      inline void setPixel(long x, float n)
      {
        buf[x] = n;
      }
    };
    // ------------------------
    class Image128x64 {
     private:
      Line128 *buf;
     public:
      Image128x64();
      Image128x64(const Image128x64& r);
      virtual ~Image128x64();
      Image128x64& operator=(const Image128x64& r);
      inline Line128& operator[](long n)
      {
        return buf[n];
      }
    };
    // ------------------------
    double  monitorGamma;
    double  ditherLimit;
    double  ditherScale;
    int     ditherMode;
    bool    luminance1BitMode;
   private:
    Image128x64 resizedImage;
    Image128x64 ditherErrorImage;
    int     *ditheredImage;
    int     colorTable[4];
    float   ditherYTable[9];
    float   errorYTable[9];
    // ----------------
    static void pixelStoreCallback(void *, int, int, float, float, float);
    void checkParameters();
    void ditherLine(long yc);
   public:
    P4FLI_MultiColorChar();
    virtual ~P4FLI_MultiColorChar();
    virtual bool processImage(PRGData& prgData, unsigned int& prgEndAddr,
                              const char *infileName,
                              YUVImageConverter& imgConv,
                              Plus4Emu::ConfigurationDB& config);
  };

}       // namespace Plus4FLIConv

#endif  // P4FLICONV_MCCHAR_HPP

