
// p4fliconv: high resolution interlaced FLI converter utility
// Copyright (C) 2007-2008 Istvan Varga <istvanv@users.sourceforge.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// The Plus/4 program files generated by this utility are not covered by the
// GNU General Public License, and can be used, modified, and distributed
// without any restrictions.

#ifndef P4FLICONV_P4SLIB_HPP
#define P4FLICONV_P4SLIB_HPP

#include "plus4emu.hpp"
#include "prgdata.hpp"

namespace Plus4FLIConv {

  /*!
   * Save the converted image from 'prgData' as a PixelShop P4S format file.
   * FLI images with interlace or non-zero X shift cannot be saved in this
   * format. On error, std::exception is thrown.
   */
  void writeP4SFile(const char *fileName, PRGData& prgData);

  /*!
   * Read a PixelShop P4S format file to 'prgData'. The start and end address
   * of the raw image data are stored in 'prgStartAddr' and 'prgEndAddr'.
   * If the file is not in P4S format, false is returned. On error,
   * std::exception is thrown.
   */
  bool readP4SFile(const char *fileName, PRGData& prgData,
                   unsigned int& prgStartAddr, unsigned int& prgEndAddr);

}       // namespace Plus4FLIConv

#endif  // P4FLICONV_P4SLIB_HPP

